import os
from genesys.app.services import svn_service
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL, TEMPLATE_FILES_DIR
from genesys.app.utils import config_helpers
from configparser import ConfigParser
from genesys.app.config import (SVN_PARENT_PATH,
                                SVN_PARENT_URL,
                                LOGIN_NAME)

def create_task_file(project_name, base_file_directory, base_svn_directory, all_persons):
    "creates file tasks"
    acl_parser = ConfigParser()
    svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name.replace(' ', '_').lower(), 'conf/authz')
    blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
    all_users = list()
    for person in all_persons:
        if person[LOGIN_NAME] != '':
            all_users.append(person[LOGIN_NAME])

    config_helpers.load_config(svn_authz_path, acl_parser)
    print(blend_file_url)

    file_folder_url = os.path.dirname(blend_file_url)
    file_name = os.path.basename(base_file_directory).rsplit('.', 1)[0]
    file_map_folder_url = os.path.join(file_folder_url, 'maps', file_name)
    base_map_svn_directory = os.path.join(os.path.dirname(base_svn_directory), 'maps', file_name)

    if not svn_service.is_svn_url(file_folder_url):
        svn_service.svn_make_dirs(file_folder_url, log_message=f'created {file_folder_url}')
    if not svn_service.is_svn_url(file_map_folder_url):
        svn_service.svn_make_dirs(file_map_folder_url, log_message=f'created {file_map_folder_url}')
    if not svn_service.is_svn_url(blend_file_url):
        svn_service.svn_import(path=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'),
                                    repo_url=blend_file_url,
                                    log_message=f'created {blend_file_url}')
                                   
    create_new_task_acl(all_users=all_users,
                        base_svn_directory=base_svn_directory,
                        acl_parser=acl_parser)
    config_helpers.write_config(svn_authz_path, acl_parser)

def create_new_task_acl(all_users:list, base_svn_directory:str, acl_parser):
    "create svn access entry for task file and set all users to no acces"
    file_name = os.path.basename(base_svn_directory).rsplit('.', 1)[0]
    base_map_svn_directory = os.path.join(os.path.dirname(base_svn_directory), 'maps', file_name)
    if base_svn_directory in acl_parser:
        pass
    else:
        for user in all_users:
            if base_svn_directory in acl_parser:
                acl_parser.set(base_svn_directory, user, '')
                acl_parser.set(base_map_svn_directory, user, '')
            else:
                acl_parser[base_svn_directory] = {
                    '@admin':'rw',
                    user:''
                }
                acl_parser[base_map_svn_directory] = {
                    '@admin':'rw',
                    user:''
                }

def delete_task_file(project_name, base_file_directory):
    #FIXME  remove acl
    blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
    if svn_service.is_svn_url(blend_file_url):
        svn_service.svn_delete(blend_file_url, log_message=f'created {blend_file_url}')


from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL
from genesys.app.blueprints.project.utils import create_project_folders
from genesys.app.services import svn_service, files_service, queue_store
import os
import shutil
from genesys.app import config

project = Blueprint('project', __name__)
api = Api(project)

def create_project(project_name):
    project_svn_folder = os.path.join(SVN_PARENT_PATH, project_name)
    project_repo_url = os.path.join(SVN_PARENT_URL, project_name)

    svn_service.create_svn_repo(project_svn_folder)
    create_project_folders(project_repo_url)
    svn_service.default_acl(project_name)

def rename_project(old_project_name, new_project_name):
    """
    rename project folder and repository and also relocates svn repo
    """
    new_project_svn_folder = os.path.join(SVN_PARENT_PATH, new_project_name)
    old_project_svn_folder = os.path.join(SVN_PARENT_PATH, old_project_name)
    # new_project_repo_url = os.path.join(SVN_PARENT_URL, new_project_name)
    # old_project_repo_url = os.path.join(svn_parent_url, old_project_name)
    shutil.move(old_project_svn_folder, new_project_svn_folder)
    # shutil.move(old_project_folder, new_project_folder)
    # svn_service.svn_relocate(new_project_repo_url, new_project_folder)

def archive_project(project_name):
    project_svn_folder = os.path.join(SVN_PARENT_PATH, project_name)
    svn_archive_folder = os.path.join(SVN_PARENT_PATH, 'archive')

    if not os.path.exists(svn_archive_folder):
        os.makedirs(svn_archive_folder)

    shutil.make_archive(base_name=os.path.join(svn_archive_folder, project_name), format='zip',
                        root_dir= project_svn_folder)
    shutil.rmtree(project_svn_folder)

class Project(Resource):
    """
    Create, rename, and delete project folder and repositories
    
    """
    def post(self, project_name):
        project_name = files_service.format_file_name(project_name)
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        if config.ENABLE_JOB_QUEUE:
            # remote = config.ENABLE_JOB_QUEUE_REMOTE
            # remote worker can not access files local to the web app
            # assert not remote or config.FS_BACKEND in ["s3", "swift"]

            queue_store.job_queue.enqueue(
                create_project,
                args=(project_name,),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            create_project(project_name)
            return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

    def put(self, project_name):
        data = request.get_json()
        old_project_name = files_service.format_file_name(data['old_project_name'])
        new_project_name = files_service.format_file_name(data['new_project_name'])
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                rename_project,
                args=(old_project_name, new_project_name),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            rename_project(old_project_name, new_project_name)
            return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

    def delete(self, project_name):
        project_name = files_service.format_file_name(project_name)
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                archive_project,
                args=(project_name,),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            archive_project(project_name)
            return jsonify(message=f'project archived', project_name=project_name)


api.add_resource(Project, '/project/<string:project_name>')




import os
from genesys.app.services import svn_service

def create_project_folders(project_repo):
    base_directories = ['edit', 'lib', 'refs', 'scenes', 'tools']
    for directory in base_directories:
        url = os.path.join(project_repo, directory)
        svn_service.svn_mkdir(url=url, log_message=f'create {url}')


from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import (SVN_PARENT_PATH,
                                SVN_PARENT_URL,
                                LOGIN_NAME)
from genesys.app.blueprints.task.utils import create_task_file, delete_task_file
from genesys.app.services import svn_service, queue_store
from genesys.app.utils import config_helpers
import os
from configparser import ConfigParser, NoSectionError
from genesys.app import config

task = Blueprint('task', __name__)
api = Api(task)

def update_svn_acl(data, project_name):
    if data['person'][LOGIN_NAME] != '':
        acl_parser = ConfigParser()
        svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name.replace(' ', '_').lower(), 'conf/authz')
        file_name = os.path.basename(data['base_svn_directory']).rsplit('.', 1)[0]
        base_map_svn_directory = os.path.join(os.path.dirname(data['base_svn_directory']), 'maps', file_name)
        config_helpers.load_config(svn_authz_path, acl_parser)
        svn_service.svn_update_acl(
            base_svn_directory=data['base_svn_directory'],
            acl_parser=acl_parser,
            person=data['person'][LOGIN_NAME],
            permission=data['permission']
        )
        #set permission for maps folders
        try:
            svn_service.svn_update_acl(
                base_svn_directory=base_map_svn_directory,
                acl_parser=acl_parser,
                person=data['person'][LOGIN_NAME],
                permission=data['permission']
            )
        except NoSectionError:
            splited_file_map_folder_url = data['base_svn_directory'].split(':', 1)
            base_file_directory = f"{splited_file_map_folder_url[0]}{splited_file_map_folder_url[1]}"
            blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
            file_folder_url = os.path.dirname(blend_file_url)
            file_name = os.path.basename(base_file_directory).rsplit('.', 1)[0]
            file_map_folder_url = os.path.join(file_folder_url, 'maps', file_name)

            if not svn_service.is_svn_url(file_map_folder_url):
                svn_service.svn_make_dirs(file_map_folder_url, log_message=f'created {file_map_folder_url}')
            acl_parser[base_map_svn_directory] = {
                '@admin':'rw',
                data['person'][LOGIN_NAME]:''
            }
            
        if data['permission'] == 'rw':
            for i in data['dependencies']:
                dependency_file_name = os.path.basename(i).rsplit('.', 1)[0]
                dependency_base_map_svn_directory = os.path.join(os.path.dirname(i), 'maps', dependency_file_name)
                svn_service.svn_update_acl(
                    base_svn_directory=i,
                    acl_parser=acl_parser,
                    person=data['person'][LOGIN_NAME],
                    permission='r'
                )
                svn_service.svn_update_acl(
                    base_svn_directory=dependency_base_map_svn_directory,
                    acl_parser=acl_parser,
                    person=data['person'][LOGIN_NAME],
                    permission='r'
                )
        elif data['permission'] == 'none':
            for i in data['dependencies']:
                dependency_file_name = os.path.basename(i).rsplit('.', 1)[0]
                dependency_base_map_svn_directory = os.path.join(os.path.dirname(i), 'maps', dependency_file_name)
                svn_service.svn_update_acl(
                    base_svn_directory=i,
                    acl_parser=acl_parser,
                    person=data['person'][LOGIN_NAME],
                    permission='d'
                )
                svn_service.svn_update_acl(
                    base_svn_directory=dependency_base_map_svn_directory,
                    acl_parser=acl_parser,
                    person=data['person'][LOGIN_NAME],
                    permission='d'
                )
        config_helpers.write_config(svn_authz_path, acl_parser)

class Task(Resource):
    def post(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        project = (data['project'])
        root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'],'')
        # replacing file tree mount point with genesys config mount point
        base_file_directory = data['base_file_directory'].split(root,1)[1]
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                create_task_file,
                args=(project_name, base_file_directory, data['base_svn_directory'], data['all_persons']),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            create_task_file(project_name=project_name,
                                base_file_directory=base_file_directory,
                                base_svn_directory=data['base_svn_directory'],
                                all_persons=data['all_persons'])
            return jsonify(message=f'task created')

    def delete(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        project = (data['project'])
        root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'],'')
        # replacing file tree mount point with genesys config mount point
        base_file_directory = data['base_file_directory'].split(root,1)[1]
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                delete_task_file,
                args=(project_name, base_file_directory),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            delete_task_file(project_name, base_file_directory, task_type=data['task_type'])
            return jsonify(message=f'task deleted')


class Task_File_Access_Control(Resource):
    def put(self, project_name):
        data = request.get_json()
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                update_svn_acl,
                args=(data, project_name),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            update_svn_acl(data, project_name)
            project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
            return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)


api.add_resource(Task, '/task/<string:project_name>')
api.add_resource(Task_File_Access_Control, '/task_acl/<string:project_name>')


import subprocess
import os
from configparser import ConfigParser, NoSectionError
from genesys.app.utils import config_helpers
from genesys.app.config import SVN_PARENT_PATH

def create_svn_repo(project_svn_repo_path):
    subprocess.run(['svnadmin', 'create', project_svn_repo_path], stdout=None)

def svn_check_out(project_repo_url, project_folder):
    subprocess.run(['svn', 'co', project_repo_url, project_folder], stdout=subprocess.DEVNULL)

def svn_commit_all(project_folder, svn_commit_message):
    subprocess.run(['svn', 'cleanup', project_folder], stdout=subprocess.DEVNULL)
    subprocess.run(['svn', 'add', project_folder, '--force'], stdout=subprocess.DEVNULL)
    subprocess.run(['svn', 'cleanup', project_folder], stdout=subprocess.DEVNULL)
    subprocess.run(['svn', 'commit', project_folder, '-m', svn_commit_message], stdout=subprocess.DEVNULL)

def svn_relocate(new_url, working_path):
    subprocess.run(['svn', 'relocate', new_url, working_path], stdout=subprocess.DEVNULL)

def svn_delete(url, log_message):
    subprocess.run(['svn', 'delete', '-m', log_message, url], stdout=subprocess.DEVNULL)
    url_dir_name = os.path.dirname(url)
    proc = subprocess.run(['svn', 'list', url_dir_name], stdout=subprocess.PIPE)
    if is_svn_url(url_dir_name) and not bool(proc.stdout.decode()):
        svn_delete(url_dir_name, log_message='deleted')

def is_svn_url(url):
    proc = subprocess.run(['svn', 'list', url], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    return not bool(proc.returncode)

def svn_rename(old_url, new_url, log_message):
    subprocess.run(['svn', 'rename', '-m', log_message, '--parents', old_url, new_url], stdout=subprocess.DEVNULL)
    url_dir_name = os.path.dirname(old_url)
    proc = subprocess.run(['svn', 'list', url_dir_name], stdout=subprocess.PIPE)
    if is_svn_url(url_dir_name) and not bool(proc.stdout.decode()):
        svn_delete(url_dir_name, log_message='deleted')

def svn_mkdir(url, log_message):
    subprocess.run(['svn', 'mkdir', '-m', log_message, url], stdout=subprocess.DEVNULL)

def svn_make_dirs(url, log_message):
    subprocess.run(['svn', 'mkdir', '-m', log_message, '--parents', url], stdout=subprocess.DEVNULL)

def svn_import(path, repo_url, log_message):
    subprocess.run(['svn', 'import', '-m', log_message, path, repo_url], stdout=subprocess.DEVNULL)

def default_acl(project_name):
    svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name.replace(' ', '_').lower(), 'conf/authz')
    acl_parser = ConfigParser()
    acl_parser['groups'] = {
                'admin':'suser',
                'maps':'',
                'edit':'',
            }
    #TODO set root to read access only
    acl_parser['/'] = {
                '*':'r',
                '@admin':'rw',
            }
    config_helpers.write_config(svn_authz_path, acl_parser)

def svn_update_acl(base_svn_directory, acl_parser, person, permission):
    if permission == 'rw':
        acl_parser.set(base_svn_directory, person, permission)
    elif permission == 'r':
        if acl_parser.get(base_svn_directory, person) == '':
            acl_parser.set(base_svn_directory, person, permission)
    elif permission == 'd':
        if acl_parser.get(base_svn_directory, person) != 'rw':
            acl_parser.set(base_svn_directory, person, '')
    elif permission == 'none':
        acl_parser.set(base_svn_directory, person, '')

def svn_read_file(url):
    proc = subprocess.run(['svn', 'cat', url], stdout=subprocess.PIPE)
    file = proc.stdout.decode("utf-8")
    return file

def svn_propset(url, prop_name, prop_val):
    subprocess.run(['svn', 'propset', prop_name, prop_val, url], stdout=subprocess.DEVNULL)